@@ -35,9 +35,11 @@ enum {
   BABL_TYPE_INTEGER,
   BABL_TYPE_FLOAT,
   BABL_SAMPLING,
+  BABL_TRC,
   BABL_COMPONENT,
   BABL_MODEL,
   BABL_FORMAT,
+  BABL_SPACE,
 
   BABL_CONVERSION,
   BABL_CONVERSION_LINEAR,
@@ -57,6 +59,8 @@ enum {
 
 #include "babl-type.h"
 #include "babl-sampling.h"
+#include "babl-trc.h"
+#include "babl-space.h"
 #include "babl-component.h"
 #include "babl-model.h"
 #include "babl-format.h"
@@ -75,6 +79,8 @@ typedef union _Babl
   BablInstance      instance;
   BablType          type;
   BablSampling      sampling;
+  BablSpace         space;
+  BablTRC           trc;
   BablComponent     component;
   BablModel         model;
   BablFormat        format;
