@@ -115,6 +115,7 @@ babl_fish_reference (const Babl *source,
 #ifndef HAVE_TLS
       free (name);
 #endif
+      _babl_fish_rig_dispatch (babl);
       return babl;
     }
 
@@ -124,7 +125,7 @@ babl_fish_reference (const Babl *source,
   babl_assert (source->class_type == BABL_FORMAT);
   babl_assert (destination->class_type == BABL_FORMAT);
 
-  babl = babl_malloc (sizeof (BablFishReference) +
+  babl = babl_calloc (1, sizeof (BablFishReference) +
                       strlen (name) + 1);
   babl->class_type    = BABL_FISH_REFERENCE;
   babl->instance.id   = babl_fish_get_id (source, destination);
@@ -133,11 +134,11 @@ babl_fish_reference (const Babl *source,
   babl->fish.source      = source;
   babl->fish.destination = destination;
 
-  babl->fish.processings = 0;
   babl->fish.pixels      = 0;
   babl->fish.error       = 0.0;  /* assuming the provided reference conversions for types
                                     and models are as exact as possible
                                   */
+  _babl_fish_rig_dispatch (babl);
 
   /* Since there is not an already registered instance by the required
    * name, inserting newly created class into database.
@@ -153,7 +154,7 @@ babl_fish_reference (const Babl *source,
 static void
 convert_to_double (BablFormat      *source_fmt,
                    const char      *source_buf,
-                   char            *source_double_buf,
+                   char            *double_buf,
                    int              n)
 {
   int        i;
@@ -171,36 +172,54 @@ convert_to_double (BablFormat      *source_fmt,
     (dst_img->type[0]->bits / 8) * source_fmt->model->components;
   dst_img->stride[0] = 0;
 
-
-  src_img->data[0]   = (char *)source_buf;
   src_img->type[0]   = (BablType *) babl_type_from_id (BABL_DOUBLE);
   src_img->pitch[0]  = source_fmt->bytes_per_pixel;
   src_img->stride[0] = 0;
 
   {
-  /* i is source position */
-  for (i = 0; i < source_fmt->components; i++)
+  /* i is dest position */
+  for (i = 0; i < source_fmt->model->components; i++)
     {
       int j;
+      int found = 0;
+
+      dst_img->data[0] =
+        double_buf + (dst_img->type[0]->bits / 8) * i;
+
+      src_img->data[0] = (char *)source_buf;
 
-      src_img->type[0] = source_fmt->type[i];
       /* j is source position */
-      for (j = 0; j < source_fmt->model->components; j++)
+      for (j = 0; j < source_fmt->components; j++)
         {
-          if (source_fmt->component[i] ==
-              source_fmt->model->component[j])
-            {
-              dst_img->data[0] =
-                source_double_buf + (dst_img->type[0]->bits / 8) * j;
+          src_img->type[0] = source_fmt->type[j];
 
-              babl_process (assert_conversion_find (src_img->type[0],
-                                                    dst_img->type[0]),
-                            src_img, dst_img, n);
+          if (source_fmt->component[j] ==
+              source_fmt->model->component[i])
+            {
+              babl_conversion_process (assert_conversion_find (src_img->type[0], dst_img->type[0]),
+                                       (void*)src_img, (void*)dst_img, n);
+              found = 1;
               break;
             }
+
+          src_img->data[0] += src_img->type[0]->bits / 8;
         }
 
-      src_img->data[0] += src_img->type[0]->bits / 8;
+      if (!found)
+        {
+          char *dst_ptr = dst_img->data[0];
+          double value;
+
+          value = source_fmt->model->component[i]->instance.id == BABL_ALPHA ? 1.0 : 0.0;
+
+          for (j = 0; j < n; j++)
+            {
+              double *dst_component = (double *) dst_ptr;
+
+              *dst_component = value;
+              dst_ptr += dst_img->pitch[0];
+            }
+        }
     }
   }
   babl_free (src_img);
@@ -247,9 +266,9 @@ convert_from_double (BablFormat *destination_fmt,
               src_img->data[0] =
                 destination_double_buf + (src_img->type[0]->bits / 8) * j;
 
-              babl_process (assert_conversion_find (src_img->type[0],
-                                                    dst_img->type[0]),
-                            src_img, dst_img, n);
+              babl_conversion_process (assert_conversion_find (src_img->type[0],
+                                       dst_img->type[0]),
+                                       (void*)src_img, (void*)dst_img, n);
               break;
             }
         }
@@ -286,9 +305,9 @@ ncomponent_convert_to_double (BablFormat       *source_fmt,
 
   dst_img->data[0] = source_double_buf;
 
-  babl_process (
+  babl_conversion_process (
     assert_conversion_find (src_img->type[0], dst_img->type[0]),
-    src_img, dst_img,
+    (void*)src_img, (void*)dst_img,
     n * source_fmt->components);
   babl_free (src_img);
   babl_free (dst_img);
@@ -320,9 +339,9 @@ ncomponent_convert_from_double (BablFormat *destination_fmt,
   dst_img->type[0] = destination_fmt->type[0];
   src_img->data[0] = destination_double_buf;
 
-  babl_process (
+  babl_conversion_process (
     assert_conversion_find (src_img->type[0], dst_img->type[0]),
-    src_img, dst_img,
+    (void*)src_img, (void*)dst_img,
     n * destination_fmt->components);
 
   dst_img->data[0] += dst_img->type[0]->bits / 8;
@@ -369,7 +388,7 @@ process_to_n_component (const Babl  *babl,
   return 0;
 }
 
-static int
+static void
 process_same_model (const Babl  *babl,
                     const char *source,
                     char       *destination,
@@ -419,14 +438,14 @@ process_same_model (const Babl  *babl,
       );
     }
   babl_free (double_buf);
-  return 0;
 }
 
-long
+void
 babl_fish_reference_process (const Babl *babl,
                              const char *source,
                              char       *destination,
-                             long        n)
+                             long        n,
+                             void       *data)
 {
   void *source_double_buf;
   void *rgba_double_buf;
@@ -440,11 +459,15 @@ babl_fish_reference_process (const Babl *babl,
       (BABL (babl->fish.source)->format.space ==
        BABL (babl->fish.destination)->format.space)
       )
-    return process_same_model (babl, source, destination, n);
+  {
+    process_same_model (babl, source, destination, n);
+    return;
+  }
 
   if (babl_format_is_format_n (BABL (babl->fish.destination)))
   {
-    return process_to_n_component (babl, source, destination, n);
+    process_to_n_component (babl, source, destination, n);
+    return;
   }
 
   source_double_buf = babl_malloc (sizeof (double) * n *
@@ -477,14 +500,14 @@ babl_fish_reference_process (const Babl *babl,
       );
     if (conv->class_type == BABL_CONVERSION_PLANAR)
       {
-        babl_process (
+        babl_conversion_process (
           conv,
-          source_image, rgba_image,
+          (void*)source_image, (void*)rgba_image,
           n);
       }
     else if (conv->class_type == BABL_CONVERSION_LINEAR)
       {
-        babl_process (
+        babl_conversion_process (
           conv,
           source_double_buf, rgba_double_buf,
           n);
@@ -513,14 +536,14 @@ babl_fish_reference_process (const Babl *babl,
       BABL (babl->fish.destination)->format.model);
     if (conv->class_type == BABL_CONVERSION_PLANAR)
       {
-        babl_process (
+        babl_conversion_process (
           conv,
-          rgba_image, destination_image,
+          (void*)rgba_image, (void*)destination_image,
           n);
       }
     else if (conv->class_type == BABL_CONVERSION_LINEAR)
       {
-        babl_process (
+        babl_conversion_process (
           conv,
           rgba_double_buf, destination_double_buf,
           n);
@@ -542,5 +565,4 @@ babl_fish_reference_process (const Babl *babl,
   babl_free (destination_double_buf);
   babl_free (rgba_double_buf);
   babl_free (source_double_buf);
-  return n;
 }
