@@ -643,8 +643,6 @@ babl_format_loss (const Babl *babl)
 
   loss = babl_rel_avg_error (clipped, test, test_pixels * 4);
 
-  fish_to->fish.processings   -= 2;
-  fish_from->fish.processings -= 2;
   fish_to->fish.pixels        -= test_pixels * 2;
   fish_from->fish.pixels      -= test_pixels * 2;
 
@@ -739,6 +737,8 @@ babl_format_with_space (const char *name, const Babl *space)
     sprintf (new_name, "%s-%s", name, babl_get_name ((Babl*)space));
 
     ret = babl_db_exist_by_name (db, new_name);
+
+    babl_free (new_name);
     if (ret)
       return ret;
 
