@@ -22,12 +22,8 @@
 long
 babl_ticks     (void);
 
-long
-babl_process_cost (long ticks_start,
-                   long ticks_end);
-
 double
-babl_rel_avg_error (double *imgA,
-                    double *imgB,
-                    long    samples);
+babl_rel_avg_error (const double *imgA,
+                    const double *imgB,
+                    long          samples);
 #endif
