@@ -156,6 +156,15 @@ char *babl_icc_get_key (const char *icc_data,
  */
 const Babl * babl_format            (const char *name);
 
+/**
+ * babl_format_exists:
+ *
+ * Returns 1 if the provided format name is known by babl or 0 if it is
+ * not. Can also be used to verify that specific extension formats are
+ * available (though this can also be inferred from the version of babl).
+ */
+int babl_format_exists              (const char *name);
+
 /**
  * babl_format_with_space:
  *
@@ -217,6 +226,15 @@ long         babl_process   (const Babl *babl_fish,
                              long  n);
 
 
+long         babl_process_rows (const Babl *babl_fish,
+                                const void *source,
+                                int         source_stride,
+                                void       *dest,
+                                int         dest_stride,
+                                long        n,
+                                int         rows);
+
+
 /**
  * babl_get_name:
  *
