@@ -22,8 +22,6 @@
 #include "babl-classes.h"
 #include "babl.h"
 #include "babl-ids.h"
-#include "util.h"
-#include "rgb-constants.h"
 #include "math.h"
 #include "babl-base.h"
 
@@ -109,11 +107,17 @@ models (void)
 }
 
 
-static long
-rgba_to_graya (char *src,
+static void
+rgba_to_graya (Babl *conversion,
+               char *src,
                char *dst,
                long  n)
 {
+  const Babl *space = babl_conversion_get_source_space (conversion);
+  double RGB_LUMINANCE_RED   = space->space.RGBtoXYZ[3];
+  double RGB_LUMINANCE_GREEN = space->space.RGBtoXYZ[4];
+  double RGB_LUMINANCE_BLUE  = space->space.RGBtoXYZ[5];
+
   while (n--)
     {
       double red, green, blue;
@@ -134,14 +138,19 @@ rgba_to_graya (char *src,
       src += sizeof (double) * 4;
       dst += sizeof (double) * 2;
     }
-  return n;
 }
 
-static long
-rgba_to_gray (char *src,
+static void
+rgba_to_gray (Babl *conversion,
+              char *src,
               char *dst,
               long  n)
 {
+  const Babl *space = babl_conversion_get_source_space (conversion);
+  double RGB_LUMINANCE_RED   = space->space.RGBtoXYZ[3];
+  double RGB_LUMINANCE_GREEN = space->space.RGBtoXYZ[4];
+  double RGB_LUMINANCE_BLUE  = space->space.RGBtoXYZ[5];
+
   while (n--)
     {
       double red, green, blue;
@@ -160,12 +169,11 @@ rgba_to_gray (char *src,
       src += sizeof (double) * 4;
       dst += sizeof (double) * 1;
     }
-  return n;
 }
 
-
-static long
-rgb_to_gray_2_2 (int    src_bands,
+static void
+rgb_to_gray_2_2 (Babl  *conversion,
+                 int    src_bands,
                  char **src,
                  int   *src_pitch,
                  int    dst_bands,
@@ -173,6 +181,12 @@ rgb_to_gray_2_2 (int    src_bands,
                  int   *dst_pitch,
                  long   n)
 {
+  const Babl *space = babl_conversion_get_destination_space (conversion);
+  const Babl *trc = space->space.trc[0];
+  double RGB_LUMINANCE_RED   = space->space.RGBtoXYZ[3];
+  double RGB_LUMINANCE_GREEN = space->space.RGBtoXYZ[4];
+  double RGB_LUMINANCE_BLUE  = space->space.RGBtoXYZ[5];
+
   BABL_PLANAR_SANITY
   while (n--)
     {
@@ -187,22 +201,22 @@ rgb_to_gray_2_2 (int    src_bands,
       else
         alpha = 1.0;
 
-      luminance = red * RGB_LUMINANCE_RED +
+      luminance = red   * RGB_LUMINANCE_RED +    // XXX: should be taken from BablSpace
                   green * RGB_LUMINANCE_GREEN +
-                  blue * RGB_LUMINANCE_BLUE;
-      *(double *) dst[0] = linear_to_gamma_2_2 (luminance);
+                  blue  * RGB_LUMINANCE_BLUE;
+      *(double *) dst[0] = babl_trc_from_linear (trc, luminance);
 
       if (dst_bands == 2)
         *(double *) dst[1] = alpha;
 
       BABL_PLANAR_STEP
     }
-  return n;
 }
 
 
-static long
-gray_2_2_to_rgb (int    src_bands,
+static void
+gray_2_2_to_rgb (Babl *conversion,
+                 int    src_bands,
                  char **src,
                  int   *src_pitch,
                  int    dst_bands,
@@ -210,6 +224,9 @@ gray_2_2_to_rgb (int    src_bands,
                  int   *dst_pitch,
                  long   n)
 {
+  const Babl *space = babl_conversion_get_source_space (conversion);
+  const Babl *trc = space->space.trc[0];
+
   BABL_PLANAR_SANITY
   while (n--)
     {
@@ -217,7 +234,7 @@ gray_2_2_to_rgb (int    src_bands,
       double red, green, blue;
       double alpha;
 
-      luminance = gamma_2_2_to_linear (*(double *) src[0]);
+      luminance = babl_trc_to_linear (trc, *(double *) src[0]);
       red       = luminance;
       green     = luminance;
       blue      = luminance;
@@ -235,13 +252,11 @@ gray_2_2_to_rgb (int    src_bands,
 
       BABL_PLANAR_STEP
     }
-  return n;
 }
 
-
-
-static long
-graya_to_rgba (char *src,
+static void
+graya_to_rgba (Babl *conversion,
+               char *src,
                char *dst,
                long  n)
 {
@@ -265,12 +280,12 @@ graya_to_rgba (char *src,
       src += sizeof (double) * 2;
       dst += sizeof (double) * 4;
     }
-  return n;
 }
 
 
-static long
-gray_to_rgba (char *src,
+static void
+gray_to_rgba (Babl *conversion,
+              char *src,
               char *dst,
               long  n)
 {
@@ -292,11 +307,11 @@ gray_to_rgba (char *src,
       src += sizeof (double) * 1;
       dst += sizeof (double) * 4;
     }
-  return n;
 }
 
-static long
-gray_alpha_premultiplied_to_rgba (int    src_bands,
+static void
+gray_alpha_premultiplied_to_rgba (Babl   *conversion,
+                                  int    src_bands,
                                   char **src,
                                   int   *src_pitch,
                                   int    dst_bands,
@@ -328,12 +343,12 @@ gray_alpha_premultiplied_to_rgba (int    src_bands,
       *(double *) dst[3] = alpha;
       BABL_PLANAR_STEP
     }
-  return n;
 }
 
 
-static long
-rgba_to_gray_alpha_premultiplied (int    src_bands,
+static void
+rgba_to_gray_alpha_premultiplied (Babl   *conversion,
+                                  int    src_bands,
                                   char **src,
                                   int   *src_pitch,
                                   int    dst_bands,
@@ -341,6 +356,11 @@ rgba_to_gray_alpha_premultiplied (int    src_bands,
                                   int   *dst_pitch,
                                   long   n)
 {
+  const Babl *space = babl_conversion_get_source_space (conversion);
+  double RGB_LUMINANCE_RED   = space->space.RGBtoXYZ[3];
+  double RGB_LUMINANCE_GREEN = space->space.RGBtoXYZ[4];
+  double RGB_LUMINANCE_BLUE  = space->space.RGBtoXYZ[5];
+
   BABL_PLANAR_SANITY;
   assert (src_bands == 4);
   assert (dst_bands == 2);
@@ -363,11 +383,11 @@ rgba_to_gray_alpha_premultiplied (int    src_bands,
       *(double *) dst[1] = alpha;
       BABL_PLANAR_STEP
     }
-  return n;
 }
 
-static long
-non_premultiplied_to_premultiplied (int    src_bands,
+static void
+non_premultiplied_to_premultiplied (Babl  *conversion,
+                                    int    src_bands,
                                     char **src,
                                     int   *src_pitch,
                                     int    dst_bands,
@@ -391,11 +411,11 @@ non_premultiplied_to_premultiplied (int    src_bands,
 
       BABL_PLANAR_STEP
     }
-  return n;
 }
 
-static long
-premultiplied_to_non_premultiplied (int    src_bands,
+static void
+premultiplied_to_non_premultiplied (Babl  *conversion,
+                                    int    src_bands,
                                     char **src,
                                     int   *src_pitch,
                                     int    dst_bands,
@@ -426,14 +446,20 @@ premultiplied_to_non_premultiplied (int    src_bands,
 
       BABL_PLANAR_STEP
     }
-  return n;
 }
 
-static long
-rgba2gray_gamma_2_2_premultiplied (char *src,
+static void
+rgba2gray_gamma_2_2_premultiplied (Babl *conversion,
+                                   char *src,
                                    char *dst,
                                    long  n)
 {
+  const Babl *space = babl_conversion_get_destination_space (conversion);
+  const Babl *trc = space->space.trc[0];
+  double RGB_LUMINANCE_RED   = space->space.RGBtoXYZ[3];
+  double RGB_LUMINANCE_GREEN = space->space.RGBtoXYZ[4];
+  double RGB_LUMINANCE_BLUE  = space->space.RGBtoXYZ[5];
+
   while (n--)
     {
       double red   = ((double *) src)[0];
@@ -447,7 +473,7 @@ rgba2gray_gamma_2_2_premultiplied (char *src,
       luminance = red * RGB_LUMINANCE_RED +
                   green * RGB_LUMINANCE_GREEN +
                   blue * RGB_LUMINANCE_BLUE;
-      luma = linear_to_gamma_2_2 (luminance);
+      luma = babl_trc_from_linear (trc, luminance);
 
       ((double *) dst)[0] = luma * alpha;
       ((double *) dst)[1] = alpha;
@@ -455,23 +481,30 @@ rgba2gray_gamma_2_2_premultiplied (char *src,
       src += 4 * sizeof (double);
       dst += 2 * sizeof (double);
     }
-  return n;
 }
 
 
-static long
-gray_gamma_2_2_premultiplied2rgba (char *src,
+static void
+gray_gamma_2_2_premultiplied2rgba (Babl *conversion,
+                                   char *src,
                                    char *dst,
                                    long  n)
 {
+  const Babl *space = babl_conversion_get_destination_space (conversion);
+  const Babl *trc = space->space.trc[0];
+
   while (n--)
     {
       double luma  = ((double *) src)[0];
       double alpha = ((double *) src)[1];
       double luminance;
 
-      luma      = luma / alpha;
-      luminance = gamma_2_2_to_linear (luma);
+      if (alpha > BABL_ALPHA_THRESHOLD)
+        luma      = luma / alpha;
+      else
+        luma      = 0.0;
+
+      luminance = babl_trc_to_linear (trc, luma);
 
       ((double *) dst)[0] = luminance;
       ((double *) dst)[1] = luminance;
@@ -481,7 +514,6 @@ gray_gamma_2_2_premultiplied2rgba (char *src,
       src += 2 * sizeof (double);
       dst += 4 * sizeof (double);
     }
-  return n;
 }
 
 
