@@ -25,8 +25,9 @@
 #include "babl-internal.h"
 #include "babl-base.h"
 
-static inline long
-convert_double_u32_scaled (double   min_val,
+static inline void
+convert_double_u32_scaled (BablConversion *c,
+                           double   min_val,
                            double   max_val,
                            uint32_t min,
                            uint32_t max,
@@ -52,11 +53,11 @@ convert_double_u32_scaled (double   min_val,
       dst              += dst_pitch;
       src              += src_pitch;
     }
-  return n;
 }
 
-static inline long
-convert_u32_double_scaled (double   min_val,
+static inline void
+convert_u32_double_scaled (BablConversion *c,
+                           double   min_val,
                            double   max_val,
                            uint32_t min,
                            uint32_t max,
@@ -82,29 +83,28 @@ convert_u32_double_scaled (double   min_val,
       dst              += dst_pitch;
       src              += src_pitch;
     }
-  return n;
 }
 
 #define MAKE_CONVERSIONS(name, min_val, max_val, min, max)      \
-  static long \
-  convert_ ## name ## _double (void *src, \
+  static void \
+  convert_ ## name ## _double (BablConversion *c, void *src, \
                                void *dst, \
                                int src_pitch, \
                                int dst_pitch, \
                                long n)                               \
   { \
-    return convert_u32_double_scaled (min_val, max_val, min, max, \
-                                      src, dst, src_pitch, dst_pitch, n); \
+    convert_u32_double_scaled (c, min_val, max_val, min, max, \
+                               src, dst, src_pitch, dst_pitch, n); \
   }                                                               \
-  static long \
-  convert_double_ ## name (void *src, \
+  static void \
+  convert_double_ ## name (BablConversion *c, void *src, \
                            void *dst, \
                            int src_pitch, \
                            int dst_pitch, \
                            long n)                                 \
   { \
-    return convert_double_u32_scaled (min_val, max_val, min, max, \
-                                      src, dst, src_pitch, dst_pitch, n); \
+    convert_double_u32_scaled (c, min_val, max_val, min, max, \
+                               src, dst, src_pitch, dst_pitch, n); \
   }
 
 MAKE_CONVERSIONS (u32, 0.0, 1.0, 0, UINT32_MAX)
