@@ -80,9 +80,9 @@ components (void)
   babl_component_new ("CIE b", "chroma", NULL);
   babl_component_new ("CIE C(ab)", "chroma", NULL);
   babl_component_new ("CIE H(ab)", "chroma", NULL);
-  /* babl_component_new ("CIE X", NULL);
+  babl_component_new ("CIE X", NULL);
   babl_component_new ("CIE Y", NULL);
-  babl_component_new ("CIE Z", NULL);*/
+  babl_component_new ("CIE Z", NULL);
 }
 
 static void
@@ -117,12 +117,21 @@ models (void)
     babl_component ("CIE H(ab)"),
     babl_component ("A"),
     NULL);
-  /*babl_model_new (
+
+  babl_model_new (
     "name", "CIE XYZ",
     babl_component ("CIE X"),
     babl_component ("CIE Y"),
     babl_component ("CIE Z"),
-    NULL);*/
+    NULL);
+
+  babl_model_new (
+    "name", "CIE XYZ alpha",
+    babl_component ("CIE X"),
+    babl_component ("CIE Y"),
+    babl_component ("CIE Z"),
+    babl_component ("A"),
+    NULL);
 }
 
 static void  rgbcie_init (void);
@@ -218,7 +227,7 @@ LAB_to_XYZ (double L,
 }
 
 static void
-rgba_to_lab (const Babl *conversion,char *src,
+rgba_to_xyz (const Babl *conversion,char *src,
              char *dst,
              long  n)
 {
@@ -228,12 +237,72 @@ rgba_to_lab (const Babl *conversion,char *src,
       double RGB[3]  = {((double *) src)[0],
                         ((double *) src)[1],
                         ((double *) src)[2] };
+      babl_space_to_xyz (space, RGB, (double*)dst);
+
+      src += sizeof (double) * 4;
+      dst += sizeof (double) * 3;
+    }
+}
+
+static void
+xyz_to_rgba (const Babl *conversion,char *src,
+             char *dst,
+             long  n)
+{
+  const Babl *space = babl_conversion_get_destination_space (conversion);
+  while (n--)
+    {
+      babl_space_from_xyz (space, (double*)src, (double*) dst);
+      ((double *) dst)[3] = 1.0;
+      src += sizeof (double) * 3;
+      dst += sizeof (double) * 4;
+    }
+}
+
+static void
+rgba_to_xyza (const Babl *conversion,char *src,
+              char *dst,
+              long  n)
+{
+  const Babl *space = babl_conversion_get_source_space (conversion);
+  while (n--)
+    {
+      babl_space_to_xyz (space, (double*)src, (double*)dst);
+      ((double *) dst)[3] = ((double *) src)[3];
+
+      src += sizeof (double) * 4;
+      dst += sizeof (double) * 4;
+    }
+}
+
+static void
+xyza_to_rgba (const Babl *conversion,char *src,
+              char *dst,
+              long  n)
+{
+  const Babl *space = babl_conversion_get_destination_space (conversion);
+  while (n--)
+    {
+      babl_space_from_xyz (space, (double*)src, (double*) dst);
+      ((double *) dst)[3] = ((double *) src)[3];
+
+      src += sizeof (double) * 4;
+      dst += sizeof (double) * 4;
+    }
+}
+
+
+static void
+rgba_to_lab (const Babl *conversion,char *src,
+             char *dst,
+             long  n)
+{
+  const Babl *space = babl_conversion_get_source_space (conversion);
+  while (n--)
+    {
       double XYZ[3], L, a, b;
 
-      //convert RGB to XYZ
-      babl_space_to_xyz (space, RGB, XYZ);
-
-      //convert XYZ to Lab
+      babl_space_to_xyz (space, (double*)src, XYZ);
       XYZ_to_LAB (XYZ[0], XYZ[1], XYZ[2], &L, &a, &b);
 
       ((double *) dst)[0] = L;
@@ -245,6 +314,8 @@ rgba_to_lab (const Babl *conversion,char *src,
     }
 }
 
+
+
 static void
 lab_to_rgba (const Babl *conversion,char *src,
              char *dst,
@@ -258,7 +329,7 @@ lab_to_rgba (const Babl *conversion,char *src,
       double b = ((double *) src)[2];
 
       double X, Y, Z, R, G, B;
-      
+
       //convert Lab to XYZ
       LAB_to_XYZ (L, a, b, &X, &Y, &Z);
 
@@ -290,14 +361,11 @@ rgba_to_laba (const Babl *conversion,char *src,
   const Babl *space = babl_conversion_get_source_space (conversion);
   while (n--)
     {
-      double RGB[3]  = {((double *) src)[0],
-                        ((double *) src)[1],
-                        ((double *) src)[2] };
       double alpha = ((double *) src)[3];
       double XYZ[3], L, a, b;
 
       //convert RGB to XYZ
-      babl_space_to_xyz (space, RGB, XYZ);
+      babl_space_to_xyz (space, (double*)src, XYZ);
 
       //convert XYZ to Lab
       XYZ_to_LAB (XYZ[0], XYZ[1], XYZ[2], &L, &a, &b);
@@ -325,26 +393,16 @@ laba_to_rgba (const Babl *conversion,char *src,
       double b     = ((double *) src)[2];
       double alpha = ((double *) src)[3];
 
-      double X, Y, Z, R, G, B;
+      double X, Y, Z;
 
       //convert Lab to XYZ
       LAB_to_XYZ (L, a, b, &X, &Y, &Z);
 
-      //convert XYZ to RGB
-      //XYZ_to_RGB (X, Y, Z, &R, &G, &B);
-
       {
+        //convert XYZ to RGB
         double XYZ[3]  = {X,Y,Z};
-        double RGB[3];
-        babl_space_from_xyz (space, XYZ, RGB);
-        R = RGB[0];
-        G = RGB[1];
-        B = RGB[2];
+        babl_space_from_xyz (space, XYZ, (double*)dst);
       }
-
-      ((double *) dst)[0] = R;
-      ((double *) dst)[1] = G;
-      ((double *) dst)[2] = B;
       ((double *) dst)[3] = alpha;
 
       src += sizeof (double) * 4;
@@ -385,13 +443,10 @@ rgba_to_lchab (const Babl *conversion,char *src,
 
   while (n--)
     {
-      double RGB[3]  = {((double *) src)[0],
-                        ((double *) src)[1],
-                        ((double *) src)[2] };
       double XYZ[3], L, a, b, C, H;
 
       //convert RGB to XYZ
-      babl_space_to_xyz (space, RGB, XYZ);
+      babl_space_to_xyz (space, (double *)src, XYZ);
 
       //convert XYZ to Lab
       XYZ_to_LAB (XYZ[0], XYZ[1], XYZ[2], &L, &a, &b);
@@ -421,7 +476,7 @@ lchab_to_rgba (const Babl *conversion,char *src,
       double L = ((double *) src)[0];
       double C = ((double *) src)[1];
       double H = ((double *) src)[2];
-      double a, b, X, Y, Z, R, G, B;
+      double a, b, X, Y, Z;
 
       //Convert LCH(ab) to Lab
       CHab_to_ab (C, H, &a, &b);
@@ -432,17 +487,9 @@ lchab_to_rgba (const Babl *conversion,char *src,
       //Convert XYZ to RGB
       {
         double XYZ[3]  = {X,Y,Z};
-        double RGB[3];
-        babl_space_from_xyz (space, XYZ, RGB);
-        R = RGB[0];
-        G = RGB[1];
-        B = RGB[2];
+        babl_space_from_xyz (space, XYZ, (double*)dst);
       }
 
-
-      ((double *) dst)[0] = R;
-      ((double *) dst)[1] = G;
-      ((double *) dst)[2] = B;
       ((double *) dst)[3] = 1.0;
 
       src += sizeof (double) * 3;
@@ -459,14 +506,11 @@ rgba_to_lchaba (const Babl *conversion,char *src,
 
   while (n--)
     {
-      double RGB[3]  = {((double *) src)[0],
-                        ((double *) src)[1],
-                        ((double *) src)[2] };
       double alpha = ((double *) src)[3];
       double XYZ[3], L, a, b, C, H;
 
       //convert RGB to XYZ
-      babl_space_to_xyz (space, RGB, XYZ);
+      babl_space_to_xyz (space, (double*)src, XYZ);
 
       //convert XYZ to Lab
       XYZ_to_LAB (XYZ[0], XYZ[1], XYZ[2], &L, &a, &b);
@@ -496,7 +540,7 @@ lchaba_to_rgba (const Babl *conversion,char *src,
       double C     = ((double *) src)[1];
       double H     = ((double *) src)[2];
       double alpha = ((double *) src)[3];
-      double a, b, X, Y, Z, R, G, B;
+      double a, b, X, Y, Z;
 
       //Convert LCH(ab) to Lab
       CHab_to_ab (C, H, &a, &b);
@@ -507,16 +551,8 @@ lchaba_to_rgba (const Babl *conversion,char *src,
       //Convert XYZ to RGB
       {
         double XYZ[3]  = {X,Y,Z};
-        double RGB[3];
-        babl_space_from_xyz (space, XYZ, RGB);
-        R = RGB[0];
-        G = RGB[1];
-        B = RGB[2];
+        babl_space_from_xyz (space, XYZ, (double*)dst);
       }
-
-      ((double *) dst)[0] = R;
-      ((double *) dst)[1] = G;
-      ((double *) dst)[2] = B;
       ((double *) dst)[3] = alpha;
 
       src += sizeof (double) * 4;
@@ -529,61 +565,28 @@ lchaba_to_rgba (const Babl *conversion,char *src,
 
 /******** begin floating point RGB/CIE color space conversions ********/
 
-/* origin: FreeBSD /usr/src/lib/msun/src/s_cbrtf.c */
-/*
- * Conversion to float by Ian Lance Taylor, Cygnus Support, ian@cygnus.com.
- * Debugged and optimized by Bruce D. Evans.
- */
-/*
- * ====================================================
- * Copyright (C) 1993 by Sun Microsystems, Inc. All rights reserved.
- *
- * Developed at SunPro, a Sun Microsystems, Inc. business.
- * Permission to use, copy, modify, and distribute this
- * software is freely granted, provided that this notice
- * is preserved.
- * ====================================================
+/* origin: http://www.hackersdelight.org/hdcodetxt/acbrt.c.txt
+ * permissions: http://www.hackersdelight.org/permissions.htm
  */
 /* _cbrtf(x)
  * Return cube root of x
  */
 
-#include <math.h>
 #include <stdint.h>
 
-static const unsigned
-B1 = 709958130, /* B1 = (127-127.0/3-0.03306235651)*2**23 */
-B2 = 642849266; /* B2 = (127-127.0/3-24/3-0.03306235651)*2**23 */
-
-static inline float _cbrtf(float x)
+static inline float
+_cbrtf (float x)
 {
-	float r,T;
-	union {float f; uint32_t i;} u = {x};
-	uint32_t hx = u.i & 0x7fffffff;
+  union { float f; uint32_t i; } u = { x };
 
-	if (hx >= 0x7f800000)  /* cbrt(NaN,INF) is itself */
-		return x + x;
+  u.i = u.i / 4 + u.i / 16;
+  u.i = u.i + u.i / 16;
+  u.i = u.i + u.i / 256;
+  u.i = 0x2a5137a0 + u.i;
+  u.f = 0.33333333f * (2.0f * u.f + x / (u.f * u.f));
+  u.f = 0.33333333f * (2.0f * u.f + x / (u.f * u.f));
 
-	/* rough cbrt to 5 bits */
-	if (hx < 0x00800000) {  /* zero or subnormal? */
-		if (hx == 0)
-			return x;  /* cbrt(+-0) is itself */
-		u.f = x*0x1p24f;
-		hx = u.i & 0x7fffffff;
-		hx = hx/3 + B2;
-	} else
-		hx = hx/3 + B1;
-	u.i &= 0x80000000;
-	u.i |= hx;
-
-	T = u.f;
-	r = T*T*T;
-	T = T*((float)x+x+r)/(x+r+r);
-
-	r = T*T*T;
-	T = T*((float)x+x+r)/(x+r+r);
-
-	return T;
+  return u.f;
 }
 
 static inline float
@@ -713,10 +716,7 @@ rgbaf_to_Lf (const Babl *conversion,float *src,
       float b = src[2];
 
       float yr = m_1_0 * r + m_1_1 * g + m_1_2 * b;
-
-      float fy = yr > LAB_EPSILON ? _cbrtf (yr) : (LAB_KAPPA * yr + 16.0f) / 116.0f;
-
-      float L = 116.0f * fy - 16.0f;
+      float L = yr > LAB_EPSILON ? 116.0f * _cbrtf (yr) - 16 : LAB_KAPPA * yr;
 
       dst[0] = L;
 
@@ -1190,18 +1190,30 @@ conversions (void)
     "linear", Lchabaf_to_Labaf,
     NULL
   );
-  /*babl_conversion_new (
+  babl_conversion_new (
     babl_model ("RGBA"),
     babl_model ("CIE XYZ"),
-    "linear", RGB_to_XYZ,
+    "linear", rgba_to_xyz,
     NULL
   );
   babl_conversion_new (
     babl_model ("CIE XYZ"),
     babl_model ("RGBA"),
-    "linear", XYZ_to_RGB,
+    "linear", xyz_to_rgba,
     NULL
-  );*/
+  );
+  babl_conversion_new (
+    babl_model ("RGBA"),
+    babl_model ("CIE XYZ alpha"),
+    "linear", rgba_to_xyza,
+    NULL
+  );
+  babl_conversion_new (
+    babl_model ("CIE XYZ alpha"),
+    babl_model ("RGBA"),
+    "linear", xyza_to_rgba,
+    NULL
+  );
 
   rgbcie_init ();
 }
@@ -1218,8 +1230,8 @@ formats (void)
     babl_component ("CIE a"),
     babl_component ("CIE b"),
     NULL);
-    
-  /*babl_format_new (
+
+  babl_format_new (
     "name", "CIE XYZ float",
     babl_model ("CIE XYZ"),
 
@@ -1227,7 +1239,18 @@ formats (void)
     babl_component ("CIE X"),
     babl_component ("CIE Y"),
     babl_component ("CIE Z"),
-    NULL);*/
+    NULL);
+
+  babl_format_new (
+    "name", "CIE XYZ alpha float",
+    babl_model ("CIE XYZ"),
+
+    babl_type ("float"),
+    babl_component ("CIE X"),
+    babl_component ("CIE Y"),
+    babl_component ("CIE Z"),
+    babl_component ("A"),
+    NULL);
 
   babl_format_new (
     "name", "CIE Lab alpha float",
